#include <assert.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <unistd.h>
#include <syslog.h>

#include "getpeerfqdn.h"

char *getpeerfqdn(int s) {
	struct sockaddr_in peeraddr;
	int peeraddrlen = sizeof (peeraddr);
	struct hostent *peerhost;
	static char hostname[256];

	if (getpeername (s, (void*)&peeraddr, &peeraddrlen) == -1) {
		syslog(LOG_ERR, "cannot get peer name: %m");
		exit(1);
	}

	if ((peerhost = gethostbyaddr ((void*)&peeraddr.sin_addr, 
				       sizeof(peeraddr.sin_addr),
				       peeraddr.sin_family)
	    ) == NULL) {
		syslog(LOG_ERR, "cannot get peer host by addr: %m");
	}

	if (peerhost) {
		assert(strlen(peerhost->h_name) < sizeof(hostname));
		strcpy (hostname, peerhost->h_name);
	} else {
		unsigned long inaddr;
		/* This seems to be the most portable way to get
		 * a numeric representation of the internet address.
		 */

		inaddr = ntohl(peeraddr.sin_addr.s_addr);
		sprintf(hostname, "%d.%d.%d.%d",
			(int)((inaddr >> 24) & 0xFF),
			(int)((inaddr >> 16) & 0xFF),
			(int)((inaddr >>  8) & 0xFF),
			(int)((inaddr >>  0) & 0xFF));
	}
	return hostname;
}
